@@ -1,9 +1,10 @@
+use std::convert::TryFrom;
 use std::fmt;
 
 use anyhow::{anyhow, bail, Error};
 use proc_macro2::{Span, TokenStream};
 use quote::{quote, ToTokens, TokenStreamExt};
-use syn::{parse_str, Expr, ExprLit, Ident, Index, Lit, LitByteStr, Meta, MetaNameValue, Path};
+use syn::{parse_str, Ident, Lit, LitByteStr, Meta, MetaList, MetaNameValue, NestedMeta, Path};
 
 use crate::field::{bool_attr, set_option, tag_attr, Label};
 
@@ -218,17 +219,15 @@ impl Field {
                 struct #wrap_name<'a>(&'a i32);
                 impl<'a> ::core::fmt::Debug for #wrap_name<'a> {
                     fn fmt(&self, f: &mut ::core::fmt::Formatter) -> ::core::fmt::Result {
-                        let res: ::core::result::Result<#ty, _> = ::core::convert::TryFrom::try_from(*self.0);
-                        match res {
-                            Err(_) => ::core::fmt::Debug::fmt(&self.0, f),
-                            Ok(en) => ::core::fmt::Debug::fmt(&en, f),
+                        match #ty::from_i32(*self.0) {
+                            None => ::core::fmt::Debug::fmt(&self.0, f),
+                            Some(en) => ::core::fmt::Debug::fmt(&en, f),
                         }
                     }
                 }
             }
         } else {
             quote! {
-                #[allow(non_snake_case)]
                 fn #wrap_name<T>(v: T) -> T { v }
             }
         }
@@ -268,21 +267,12 @@ impl Field {
     }
 
     /// Returns methods to embed in the message.
-    pub fn methods(&self, ident: &TokenStream) -> Option<TokenStream> {
+    pub fn methods(&self, ident: &Ident) -> Option<TokenStream> {
         let mut ident_str = ident.to_string();
         if ident_str.starts_with("r#") {
-            ident_str = ident_str.split_off(2);
+            ident_str = ident_str[2..].to_owned();
         }
 
-        // Prepend `get_` for getter methods of tuple structs.
-        let get = match syn::parse_str::<Index>(&ident_str) {
-            Ok(index) => {
-                let get = Ident::new(&format!("get_{}", index.index), Span::call_site());
-                quote!(#get)
-            }
-            Err(_) => quote!(#ident),
-        };
-
         if let Ty::Enumeration(ref ty) = self.ty {
             let set = Ident::new(&format!("set_{}", ident_str), Span::call_site());
             let set_doc = format!("Sets `{}` to the provided enum value.", ident_str);
@@ -295,8 +285,8 @@ impl Field {
                     );
                     quote! {
                         #[doc=#get_doc]
-                        pub fn #get(&self) -> #ty {
-                            ::core::convert::TryFrom::try_from(self.#ident).unwrap_or(#default)
+                        pub fn #ident(&self) -> #ty {
+                            #ty::from_i32(self.#ident).unwrap_or(#default)
                         }
 
                         #[doc=#set_doc]
@@ -313,11 +303,8 @@ impl Field {
                     );
                     quote! {
                         #[doc=#get_doc]
-                        pub fn #get(&self) -> #ty {
-                            self.#ident.and_then(|x| {
-                                let result: ::core::result::Result<#ty, _> = ::core::convert::TryFrom::try_from(x);
-                                result.ok()
-                            }).unwrap_or(#default)
+                        pub fn #ident(&self) -> #ty {
+                            self.#ident.and_then(#ty::from_i32).unwrap_or(#default)
                         }
 
                         #[doc=#set_doc]
@@ -335,14 +322,11 @@ impl Field {
                     let push_doc = format!("Appends the provided enum value to `{}`.", ident_str);
                     quote! {
                         #[doc=#iter_doc]
-                        pub fn #get(&self) -> ::core::iter::FilterMap<
+                        pub fn #ident(&self) -> ::core::iter::FilterMap<
                             ::core::iter::Cloned<::core::slice::Iter<i32>>,
                             fn(i32) -> ::core::option::Option<#ty>,
                         > {
-                            self.#ident.iter().cloned().filter_map(|x| {
-                                let result: ::core::result::Result<#ty, _> = ::core::convert::TryFrom::try_from(x);
-                                result.ok()
-                            })
+                            self.#ident.iter().cloned().filter_map(#ty::from_i32)
                         }
                         #[doc=#push_doc]
                         pub fn #push(&mut self, value: #ty) {
@@ -367,7 +351,7 @@ impl Field {
 
             Some(quote! {
                 #[doc=#get_doc]
-                pub fn #get(&self) -> #ty {
+                pub fn #ident(&self) -> #ty {
                     match self.#ident {
                         #match_some
                         ::core::option::Option::None => #default,
@@ -444,24 +428,29 @@ impl Ty {
             Meta::Path(ref name) if name.is_ident("bytes") => Ty::Bytes(BytesTy::Vec),
             Meta::NameValue(MetaNameValue {
                 ref path,
-                value:
-                    Expr::Lit(ExprLit {
-                        lit: Lit::Str(ref l),
-                        ..
-                    }),
+                lit: Lit::Str(ref l),
                 ..
             }) if path.is_ident("bytes") => Ty::Bytes(BytesTy::try_from_str(&l.value())?),
             Meta::NameValue(MetaNameValue {
                 ref path,
-                value:
-                    Expr::Lit(ExprLit {
-                        lit: Lit::Str(ref l),
-                        ..
-                    }),
+                lit: Lit::Str(ref l),
                 ..
             }) if path.is_ident("enumeration") => Ty::Enumeration(parse_str::<Path>(&l.value())?),
-            Meta::List(ref meta_list) if meta_list.path.is_ident("enumeration") => {
-                Ty::Enumeration(meta_list.parse_args::<Path>()?)
+            Meta::List(MetaList {
+                ref path,
+                ref nested,
+                ..
+            }) if path.is_ident("enumeration") => {
+                // TODO(rustlang/rust#23121): slice pattern matching would make this much nicer.
+                if nested.len() == 1 {
+                    if let NestedMeta::Meta(Meta::Path(ref path)) = nested[0] {
+                        Ty::Enumeration(path.clone())
+                    } else {
+                        bail!("invalid enumeration attribute: item must be an identifier");
+                    }
+                } else {
+                    bail!("invalid enumeration attribute: only a single identifier is supported");
+                }
             }
             _ => return Ok(None),
         };
@@ -618,12 +607,8 @@ impl DefaultValue {
     pub fn from_attr(attr: &Meta) -> Result<Option<Lit>, Error> {
         if !attr.path().is_ident("default") {
             Ok(None)
-        } else if let Meta::NameValue(MetaNameValue {
-            value: Expr::Lit(ExprLit { ref lit, .. }),
-            ..
-        }) = *attr
-        {
-            Ok(Some(lit.clone()))
+        } else if let Meta::NameValue(ref name_value) = *attr {
+            Ok(Some(name_value.lit.clone()))
         } else {
             bail!("invalid default value attribute: {:?}", attr)
         }
@@ -749,7 +734,7 @@ impl DefaultValue {
                         _ => (),
                     }
                 }
-                match syn::parse_str::<Lit>(value) {
+                match syn::parse_str::<Lit>(&value) {
                     Ok(Lit::Str(_)) => (),
                     Ok(lit) => return DefaultValue::from_lit(ty, lit),
                     _ => (),
